/**
\file mathexpr.hpp
\date 25/05/2021
\author Yann Boucher
\version 1
\brief MathExpr

Ce fichier contient une fonction permettant d'exécuter une expression mathématique à partir d'une chaîne de caractères et d'une liste de variables.
        **/

#ifndef MATHEXPR_HPP
#define MATHEXPR_HPP

#include <map>
#include <exception>

//! \brief Exception lancée lors de l'évaluation d'une expression mathématique.
class MathExprException : public std::exception
{
public:
    MathExprException(const std::string& str)
        : m_what(str)
    {}

    const char * what() const noexcept override
    { return m_what.c_str(); }

private:
    std::string m_what;
};

//! Evalue une expression mathématique selon une liste de variables fournies, et retourne un entier.
//! Les opérateurs supportés sont les parenthèses, +, -, *, /, %.
//! \param expr L'expression mathématique, en tant que std::string
//! \param variables un std::map associant à chaque nom de variable une valeur entière
//! \returns La valeur de résultat de l'expression mathématique évaluée.
int eval_math(const std::string& expr, const std::map<std::string, int>& variables = {});

#endif // MATHEXPR_HPP
