#include "mooreNeighborhoodRule.hpp"

REGISTER_FACTORY_ENTRY(NeighborhoodRule, mooreNeighborhoodRule, "Moore");

mooreNeighborhoodRule::mooreNeighborhoodRule(int _radius)
{
    radius.val = _radius;

    if(_radius == 0)
        throw NeighborhoodException("Le rayon ne peut pas être égal à 0");
    // Les coordonnées sont copiées dans le vecteur
    // N(i,j) = {(k, l)|abs(k−i) ≤ r et abs(l−j) ≤ r} ;
    Coord newCord;
    for(int i = -_radius; i <= _radius; i++){
        for(int j = -_radius; j <= _radius; j++){
            newCord.x = i;
            newCord.y = j;
            format.positions.push_back(newCord);
        }
    }
}

Neighborhood mooreNeighborhoodRule::getNeighborhood(const Grid& grid, Coord pos) const
{
    Neighborhood newNeighborhood;
    // Coordonnées des voisins dans la grille
    Coord gridCoord;
    std::vector<Coord>::const_iterator it = format.positions.begin();
    for (it = format.positions.begin() ; it != format.positions.end(); ++it) {
        // Calcul des coordonnées du voisin sur la grille
        gridCoord.x = pos.x + it->x;
        gridCoord.y = pos.y + it->y;
        // Ajout du nouveau voisin <Coordonnée_relative, état>
        newNeighborhood.addNeighbor(*it , grid.get_state(gridCoord));
    }
    return newNeighborhood;
}

std::vector<NeighborhoodFormat> mooreNeighborhoodRule::getFormats() const
{
    std::vector<NeighborhoodFormat> vector;
    vector.push_back(format);
    return vector;
}

